<?php
namespace FORM4\Form4Teaser\Domain\Repository;

/***************************************************************
 *  Copyright notice
*
*  (c) 2014 form4 GmbH & Co. KG <typo3@form4.de>
*  
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 3 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * @author Thomas Grothaus <thomas.grothaus@form4.de>
 * @author Stefan Szymanski <stefan.szymanski@form4.de>
 *
 * @package TYPO3
 * @subpackage form4_teaser
 */
class LargePageRepository extends \FORM4\Form4Pages\Domain\Repository\LargePageRepository {

	
	/**
	 * @var \FORM4\Form4Teaser\Utility\ObjectUtility
	 * @inject
	 */
	protected $objectUtility;
	
	/**
	 * CategoryRepository
	 *
	 * @var \TYPO3\CMS\Extbase\Domain\Repository\CategoryRepository
	 * @inject
	 */
	protected $categoryRepository;
	
	/**
	 * @var TYPO3\CMS\Frontend\Page\PageRepository
	 * @inject
	 */
	protected $pageRepository;

	/**
	 * The QueryGenerator is needed to recursively fetch a page tree
	 *
	 * @var \TYPO3\CMS\Core\Database\QueryGenerator
	 * @inject
	 */
	protected $queryGenerator;
	
	/**
	 * @var \TYPO3\CMS\Extbase\SignalSlot\Dispatcher
	 * @inject
	 */
	protected $signalSlotDispatcher;

	/**
	 * Returns a list of LargePage Model instances depending on the given $settings
	 * 
	 * @param array $settings Settings array from the ActionController containing the Plugin Settings
	 * @return NULL|array
	 */
	public function getPagesWithFlexformContraints($settings) {
		
		$constraints = array();
		$query = $this->createQuery();
		
		$cObject = $this->configurationManager->getContentObject();
		
		// CREATE LIST OF PAGE UIDS
		$pageIds = array();
		switch ($settings['source']) {
			case 'custom':
				$pageIds = GeneralUtility::intExplode(',', $settings['customPages']);
				$settings['hideCurrentPage'] = 0;
				$settings['recursive'] = 1;
				break;

			case 'customChildren':
				$pageIds = GeneralUtility::intExplode(',', $settings['customPages']);
				$newPageIds = array();
				foreach ($pageIds as $pageId) {
					if (isset($settings['hideParentPages']) && $settings['hideParentPages'] == '1') {
						$newPageIds = array_merge($newPageIds, GeneralUtility::intExplode(',', $cObject->getTreeList(intval($pageId), $settings['recursive']), TRUE));
					} else {
						$newPageIds = array_merge($newPageIds, GeneralUtility::intExplode(',', $cObject->getTreeList(intval($pageId), $settings['recursive']), TRUE));
						$newPageIds[] = $pageId;
					}
				}
				$pageIds = $newPageIds;
				break;

			case 'thisChildren':
				$currentPageId = $GLOBALS['TSFE']->id;
				if (isset($settings['hideParentPages']) && $settings['hideParentPages'] == '1') {
					$pageIds =  GeneralUtility::intExplode(',', $cObject->getTreeList(intval($currentPageId), $settings['recursive']), TRUE);
				} else {
					$pageIds =  GeneralUtility::intExplode(',', $cObject->getTreeList(intval($currentPageId), $settings['recursive']), TRUE);
					$pageIds[] = $currentPageId;
				}
				break;
		}
		
		// FILTER: IGNORED UIDs
		if (!empty($pageIds)) {
			if (isset($settings['ignoreUids']) && !empty($settings['ignoreUids'])) {
				$ignorePagesArr = GeneralUtility::intExplode(',', $settings['ignoreUids']);
				foreach ($ignorePagesArr as $ignoreUid) {
					$deleteKey = array_search($ignoreUid,$pageIds);
					unset($pageIds[$deleteKey]);
				}
			}
			$settings['pageUids'] = $pageIds;
		} else {
			// we're looking explicit for uids by: $this->queryGenerator->getTreeList see above
			// therefore if we didn't find any uids the expected result will be no pages
			return NULL;
		}
		
		
		// SORTING
		$sortingConfig = \FORM4\Form4Teaser\Utility\ConfigurationUtility::getSorting($settings['sorting']);
		if ($sortingConfig['userFunction']) {
			$sortingConstraints = $this->objectUtility->callUserFunction($sortingConfig['userFunction'], array(&$settings, $query));
			// when the user function returns constraints, add them to the $constraint array
			if (!is_array($sortingConstraints)) {
				$sortingConstraints = array($sortingConstraints);
			}
			$constraints = array_merge($constraints, array_filter($sortingConstraints, function($constraint) {
				return ($constraint instanceof \TYPO3\CMS\Extbase\Persistence\Generic\Qom\ConstraintInterface);
			}));
		}

		// SORTING DIRECTION
		$sortingDirectionConfig = \FORM4\Form4Teaser\Utility\ConfigurationUtility::getSortingDirection($settings['sortingDirection']);
		if ($sortingDirectionConfig['userFunction']) {
			$sortingDirectionConstraints = $this->objectUtility->callUserFunction($sortingDirectionConfig['userFunction'], array(&$settings, $query));
			// when the user function returns constraints, add them to the $constraint array
			if (!is_array($sortingDirectionConstraints)) {
				$sortingDirectionConstraints = array($sortingDirectionConstraints);
			}
			$constraints = array_merge($constraints, array_filter($sortingDirectionConstraints, function($constraint) {
				return ($constraint instanceof \TYPO3\CMS\Extbase\Persistence\Generic\Qom\ConstraintInterface);
			}));
		}
		
		
		// FILTER: DOKTYPE
		if (!empty($settings['filterDoktypes'])) {
			$constraints[] = $query->in('doktype', $settings['filterDoktypes']);
		}

		// FILTER: CATEGORIES
		if (!empty($settings['categories'])){
			$this->contentObj = $this->configurationManager->getContentObject();
			$uid = $this->contentObj->data['uid'];
			$categoriesIds = $this->getCategoryUids($uid);
			$categoryConstraints = array();
			foreach ($categoriesIds as $id) {
				$categoryConstraints[] = $query->contains('categories', $id);
			}
			$constraints[] = $query->logicalOr($categoryConstraints);
		}

		// FILTER: HIDDEN FROM NAVIGATION
		if (!isset($settings['showHiddenPages']) || $settings['showHiddenPages'] == 0) {
			$constraints[] = $query->equals('nav_hide', 0);
		}		
		
		// LIMIT
		if(isset($settings['limit']) && !empty($settings['limit'])){
			$query->setLimit(intval($settings['limit']));
		}
		
		$constraints[] = $query->in('uid', $settings['pageUids']);
		
		// EXECUTE
		$query->matching($query->logicalAnd($constraints));
		if (!empty($settings['queryResultClass'])) {
// 			$result = $this->objectManager->get($settings['queryResultClass'], $query, $settings);
			$result = $query->execute(FALSE, $settings['queryResultClass'], array($settings));
		} else {
			$result = $query->execute();
		}
		
		// EXECUTE RESULT MODIFIERS
		// a modifier gets passed $result and $settings as arguments and should return an altered result
		list($result) = $this->signalSlotDispatcher->dispatch(__CLASS__, 'modifyResults', array($result, $settings));
		
		return $result;
	}
	
	/**
	 * Get category IDs of the fieldname of the plugin
	 *
	 * @param int $uid
	 * @param string $fieldname
	 **/
	protected function getCategoryUids($uid, $fieldname ='form4teaser_categories', $tablenames = 'tt_content') {
		$rows = array();
		$selectFields = 'uid_local as uid';
		$fromTables = 'sys_category_record_mm';
		$whereClause = 'uid_foreign=' . $uid. ' AND fieldname="' .$fieldname.'" AND tablenames="' .$tablenames.'"';
		$groupBy = '';
		$sorting = 'sorting_foreign';
		$limit = '';

		$rows = $GLOBALS['TYPO3_DB']->exec_SELECTgetRows($selectFields, $fromTables, $whereClause, $groupBy, $sorting, $limit);

		$categories = array();
		foreach ($rows as $row){
			$categories[] = $row['uid'];
		}

		return	$categories;
	}

}
?>
