<?php
namespace FORM4\Form4Teaser\Utility;
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 form4 GmbH & Co. KG <typo3@form4.de>
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use \TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * This class provides methods for manipulating the sorting direction of a query.
 * Its methods are intended to get registered with \FORM4\Form4Teaser\Utility\ConfigurationUtility::addSortingDirection().
 * 
 * @author Stefan Szymanski <stefan.szymanski@form4.de>
 * 
 * @package TYPO3
 * @subpackage form4_teaser
 */
class SortingDirectionsUtility {

	/**
	 * Sort ascending
	 *
	 * @param array $settings
	 * @param \TYPO3\CMS\Extbase\Persistence\QueryInterface $query
	 * @return void
	 */
	public function ascending (&$settings, \TYPO3\CMS\Extbase\Persistence\QueryInterface $query) {
		$this->setSorting($query, $settings['sortingField'], \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_ASCENDING);
	}
	
	/**
	 * Sort descending
	 *
	 * @param array $settings
	 * @param \TYPO3\CMS\Extbase\Persistence\QueryInterface $query
	 * @return void
	 */
	public function descending (&$settings, \TYPO3\CMS\Extbase\Persistence\QueryInterface $query) {
		$this->setSorting($query, $settings['sortingField'], \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_DESCENDING);
	}
	
	/**
	 * @param \TYPO3\CMS\Extbase\Persistence\QueryInterface $query
	 * @param array|string $sortingFields
	 * @param string $direction
	 */
	protected function setSorting(\TYPO3\CMS\Extbase\Persistence\QueryInterface $query, $sortingFields, $direction) {
		if (!is_array($sortingFields)) {
			$sortingFields = array($sortingFields);
		}
		$orderings = array();
		foreach ($sortingFields as $field) {
			$orderings[$field] = $direction;
		}
		$query->setOrderings($orderings);
	}
	
	/**
	 * Sort shuffled - this will not affect pagination.
	 *
	 * @param array $settings
	 * @param \TYPO3\CMS\Extbase\Persistence\Generic\Query $query
	 * @return void
	 */
	public function shuffled (&$settings, $query) {
		$settings['queryResultClass'] = 'FORM4\\Form4Teaser\\Persistence\\ShuffledQueryResult';
	}
	
	/**
	 * Sort randomly - this will make using pagination absurd, because all results get shuffled.
	 *
	 * @param array $settings
	 * @param \TYPO3\CMS\Extbase\Persistence\Generic\Query $query
	 * @return void
	 */
	public function random (&$settings, $query) {
		$settings['queryResultClass'] = 'FORM4\\Form4Teaser\\Persistence\\RandomQueryResult';
	}
	
}
?>