<?php
namespace FORM4\Form4Teaser\Utility;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 form4 GmbH & Co. KG <typo3@form4.de>
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use \TYPO3\CMS\Extbase\Utility\LocalizationUtility;
use \TYPO3\CMS\Core\Utility\GeneralUtility;
use \FORM4\Form4Teaser\Utility\ConfigurationUtility;

/**
 * Class/Function which manipulates the items-array of flexform fields.
 *
 * @author Thomas Grothaus <thomas.grothaus@form4.de>
 * @author Stefan Szymanski <stefan.szymanski@form4.de>
 *
 * @package TYPO3
 * @subpackage form4_teaser
 */
class TcaItemProcessorUtility implements \TYPO3\CMS\Core\SingletonInterface {
	/**
	 *
	 * @var string
	 */
	const _EXTKEY = 'form4_teaser';

	/**
	 * ItemProcFunc adding the registered layouts to the flexform's settings.layout field
	 *
	 * @param array $config
	 * @return array
	 */
	public function getLayouts($config) {
		foreach (ConfigurationUtility::getLayout() as $key => $data) {
			$config['items'][] = array(
				$GLOBALS['LANG']->sL($data['label']), // do not use the LocalizationUtility, because it does not support 'LLL:EXT:...'-strings
				$key,
				GeneralUtility::getFileAbsFileName($data['icon'])
			);
		}
		return $config;
	}
	
	/**
	 * ItemProcFunc adding the registered sortings to the flexform's settings.sorting field
	 *
	 * @param array $config
	 * @return array
	 */
	public function getSortings($config) {
		foreach (ConfigurationUtility::getSorting() as $key => $data) {
			$config['items'][] = array(
				$GLOBALS['LANG']->sL($data['label']),
				$key,
				GeneralUtility::getFileAbsFileName($data['icon'])
			);
		}
	}
	
	/**
	 * ItemProcFunc adding the registered sorting directions to the flexform's settings.sortingDirection field
	 *
	 * @param array $config
	 * @return array
	 */
	public function getSortingDirections($config) {
		foreach (ConfigurationUtility::getSortingDirection() as $key => $data) {
			$config['items'][] = array(
				$GLOBALS['LANG']->sL($data['label']),
				$key,
				GeneralUtility::getFileAbsFileName($data['icon'])
			);
		}
	}

	/**
	 * ItemProcFunc adding the registered doktypes to the flexform's settings.filterDoktypes field
	 *
	 * @param array $config
	 * @return array
	 */
	public function getDoktypes($config) {
		$teaserConfig = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf'][self::_EXTKEY]);
		
		$allowedDoktypes = GeneralUtility::intExplode(',', $teaserConfig['allowedDoktypes'], TRUE);
		$forbiddenDoktypes = GeneralUtility::intExplode(',', $teaserConfig['forbiddenDoktypes'], TRUE);
		$allowedDoktypes = array_filter($allowedDoktypes, function($element) use ($forbiddenDoktypes) {
			return !in_array($element, $forbiddenDoktypes);
		});
		$allowedDoktypes = implode(',', $allowedDoktypes);
		
		$processor = GeneralUtility::makeInstance('FORM4\\Form4Pages\\Utility\\ConfigurationUtility');
		return $processor->pageDoktypeItemProcFunc($config, $allowedDoktypes);
	}

}
?>