<?php
namespace FORM4\Form4Teaser\ViewHelpers\Widget\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 form4 GmbH & Co. KG <typo3@form4.de>
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script
 ***************************************************************/

/**
 * This Controller extends the original PaginateController by the ability to distinguish between different paginated plugins on the same page.
 *
 * @package TYPO3
 * @subpackage form4_teaser
 *
 * @author Thomas Grothaus <thomas.grothaus@form4.de>
 * @author Stefan Szymanski <stefan.szymanski@form4.de>
 */
class PaginateController extends \TYPO3\CMS\Fluid\ViewHelpers\Widget\Controller\PaginateController {

	/**
	 * @var integer
	 */
	protected $contentId;

	/**
	 * @var integer
	 */
	protected $navigationBlockRangeStart;

	/**
	 * @var integer
	 */
	protected $navigationBlockRangeEnd;


	/**
	 * @return void
	 */
	public function initializeAction() {
		$this->objects = $this->widgetConfiguration['objects'];
		\TYPO3\CMS\Core\Utility\ArrayUtility::mergeRecursiveWithOverrule($this->configuration, $this->widgetConfiguration['configuration'], TRUE);

		if (!isset($this->configuration['itemsPerPage']) || empty($this->configuration['itemsPerPage'])) {
			$this->configuration['itemsPerPage'] = 999; // TODO make this default value maintainable
		}
		$this->contentId = $this->configurationManager->getContentObject()->data['uid'];
		$this->numberOfPages = ceil(count($this->objects) / (int) $this->configuration['itemsPerPage']);
		$this->maximumNumberOfLinks = (int) $this->configuration['maximumNumberOfLinks'];
	}

	/**
	 * If a certain number of links should be displayed, adjust before and after
	 * amounts accordingly.
	 *
	 * @return void
	 */
	protected function calculateDisplayRange() {
		$maximumNumberOfLinks = $this->maximumNumberOfLinks;
		if ($maximumNumberOfLinks > $this->numberOfPages) {
			$maximumNumberOfLinks = $this->numberOfPages;
		}

		$delta = floor($maximumNumberOfLinks / 2);
		$this->displayRangeStart = $this->currentPage - $delta;
		$this->displayRangeEnd = $this->currentPage + $delta + ($maximumNumberOfLinks % 2 === 0 ? 1 : 0);
		if ($this->displayRangeStart < 1) {
			$this->displayRangeEnd -= $this->displayRangeStart - 1;
		}
		if ($this->displayRangeEnd > $this->numberOfPages) {
			$this->displayRangeStart -= $this->displayRangeEnd - $this->numberOfPages;
		}
		$this->displayRangeStart = (int) max($this->displayRangeStart, 1);
		$this->displayRangeEnd = (int) min($this->displayRangeEnd, $this->numberOfPages);


		if ($this->currentPage - intval($this->settings['paginationPrevious']) < 1) {
			$this->navigationBlockRangeStart = 1;
		} else {
			$this->navigationBlockRangeStart = $this->currentPage - intval($this->settings['paginationPrevious']);
		}
		if ($this->currentPage + intval($this->settings['paginationNext']) > $this->displayRangeEnd) {
			$this->navigationBlockRangeEnd = $this->displayRangeEnd;
		} else {
			$this->navigationBlockRangeEnd = $this->currentPage + intval($this->settings['paginationNext']);
		}
	}

	/**
	 * Returns an array with the keys "pages", "current", "numberOfPages", "nextPage" & "previousPage"
	 *
	 * @return array
	 */
	protected function buildPagination() {
		$pages = array();
		for ($i = $this->displayRangeStart; $i <= $this->displayRangeEnd; $i++) {
			$pages[] = array(
				'number' => $i,
				'isCurrent' => $i === $this->currentPage,
				'isInNavBlock' => (($this->navigationBlockRangeStart <= $i) && ($i <= $this->navigationBlockRangeEnd))
			);
		}
		$pagination = array(
			'pages' => $pages,
			'current' => $this->currentPage,
			'numberOfPages' => $this->numberOfPages,
			'displayRangeStart' => $this->displayRangeStart,
			'displayRangeEnd' => $this->displayRangeEnd,
			'navBlockStart' => $this->navigationBlockRangeStart,
			'navBlockEnd' => $this->navigationBlockRangeEnd,
			'beforeNavBlockStart' => $this->navigationBlockRangeStart - 1,
			'afterNavBlockEnd' => $this->navigationBlockRangeEnd + 1,
			'displayRangeEndMarker' => $this->displayRangeEnd - 2
		);
		if ($this->currentPage < $this->numberOfPages) {
			$pagination['nextPage'] = $this->currentPage + 1;
		}
		if ($this->currentPage > 1) {
			$pagination['previousPage'] = $this->currentPage - 1;
		}
		return $pagination;
	}

	/**
	 * @param string $currentPage Format: "cid:page[;cid:page[...]]" Example: "123:2;201:4;304:2"
	 * @return void
	 */
	public function indexAction($currentPage = '0:0') {
		$itemsPerPage = (int) $this->configuration['itemsPerPage'];

		if ($this->configuration['pagingVisibility'] == 'on' || ($this->configuration['pagingVisibility'] == 'auto' && count($this->objects) > $itemsPerPage)) {
			// set default value of $currentPage if necessary
			if($currentPage == '0:0'){
				$currentPage = $this->contentId . ':1';
			}
			// explode the currentPage string
			$currentPageArray = array();
			$tempCurrentPageArray = \TYPO3\CMS\Core\Utility\GeneralUtility::trimExplode(';', $currentPage, TRUE);
			foreach ($tempCurrentPageArray as $item) {
				list($cid, $page) = \TYPO3\CMS\Core\Utility\GeneralUtility::trimExplode(':', $item, TRUE);
				$currentPageArray[$cid] = $page;
			}
			// set current page, defaults to the first page
			$this->currentPage = isset($currentPageArray[$this->contentId])
				? (int) $currentPageArray[$this->contentId]
				: 1;

			$this->calculateDisplayRange();

			if ($this->currentPage > $this->numberOfPages) {
				// set $modifiedObjects to NULL if the page does not exist
				$modifiedObjects = NULL;
			} else {
				// get the objects for the current page
				// TODO I would use setOffset() and setLimit() on the query object, but using setOffset() causes a malformed SQL query string. I dunno why.
				if(is_array($this->objects)){
					$modifiedObjects = array_slice(
						$this->objects,
						$itemsPerPage * $this->currentPage - $itemsPerPage,
						$itemsPerPage
					);
				}
				if(is_object($this->objects)){
					$modifiedObjects = array_slice(
						$this->objects->toArray(),
						$itemsPerPage * $this->currentPage - $itemsPerPage,
						$itemsPerPage
					);
				}

			}
		} else {
			$this->configuration['insertAbove'] = FALSE;
			$this->configuration['insertBelow'] = FALSE;
			$modifiedObjects = $this->objects;
		}

		$this->view->assign('contentArguments', array(
			$this->widgetConfiguration['as'] => $modifiedObjects
		));
		$this->view->assign('configuration', $this->configuration);
		$this->view->assign('pagination', $this->buildPagination());
		$this->view->assign('contentId', $this->contentId);
	}
}

?>